#!/usr/bin/perl -w
use strict;

my $truthFN = shift or die;
my $systemFN = shift or die;

my $numT = 0;  # number of truth spans
my $numS = 0;  # number of system spans
my $numL = 0;  # number of correctly labeled spans
my $numU = 0;  # number of correct spans (labeled or otherwise)

my $lineNum = 0;
open T, $truthFN or die;
open S, $systemFN or die;
while (<T>) {
    chomp;
    my $truth = $_;

    $_ = <S>;
    if (not defined $_) { die "not enough lines in $systemFN\n"; }
    chomp;
    my $system = $_;

    $lineNum++;

    if ($truth =~ /^no parse!/) { die "truth has noparse! -- that's weird!\n"; }

    $truth =~ s/^[-0-9\.]+\s+//; # remove log probs, if they're there
    $truth =~ s/\s+/ /g; $truth =~ s/^ //; $truth =~ s/ $//;
    my ($truthTree, $truthRest) = parseTree($truth);
    if ($truthRest ne '') { die; }

    my %truth = ();
    computeSpans($truthTree, \%truth, 0);

    my %system = ();
    if (not ($system =~ /no parse!/)) {
        $system =~ s/^[-0-9\.]+\s+//; # remove log probs, if they're there
        $system =~ s/\s+/ /g; $system =~ s/^ //; $system =~ s/ $//;
        my ($systemTree, $systemRest) = parseTree($system);
        if ($systemRest ne '') { die; }
        computeSpans($systemTree, \%system, 0);
    }

    foreach my $sp (keys %truth) {
        foreach my $lab (keys %{$truth{$sp}}) {
            $numT++;

            if (exists $system{$sp}) {
                $numU++;
                if (exists $system{$sp}{$lab}) {
                    $numL++;
                }
            }
        }
    }
    foreach my $sp (keys %system) {
        foreach my $lab (keys %{$system{$sp}}) {
            $numS++;
        }
    }
}
close S or die;
close T or die;

my $LP = ($numL == 0) ? 0 : ($numL / $numS);
my $LR = ($numL == 0) ? 0 : ($numL / $numT);
my $LF = ($numL == 0) ? 0 : (2 * $LP * $LR / ($LP + $LR));

my $UP = ($numU == 0) ? 0 : ($numU / $numS);
my $UR = ($numU == 0) ? 0 : ($numU / $numT);
my $UF = ($numU == 0) ? 0 : (2 * $UP * $UR / ($UP + $UR));

print "# of true spans              = $numT\n";
print "# of hypothesized spans      = $numS\n";
print "# of correct spans           = $numU\n";
print "# of correctly labeled spans = $numL\n";
print "labeled precision            = $LP\n";
print "labeled recall               = $LR\n";
print "labeled f-measure            = $LF\n";
print "unlabeled precision          = $UP\n";
print "unlabeled recall             = $UR\n";
print "unlabeled f-measure          = $UF\n";

sub computeSpans {
    my ($tr, $h, $left) = @_;

    my $lab = $tr->{'lhs'};
    if (not exists $tr->{'children'}) {
        # we're a terminal
        $h->{$left . ' ' . $left}{$lab} = 1;
        return 1;
    }

    # we're a non-terminal
    my $width = 0;
    foreach my $c (@{$tr->{'children'}}) {
        $width += computeSpans($c, $h, $left+$width);
    }
    $h->{$left . ' ' . ($left+$width)}{$lab} = 1;

    return $width;
}

sub parseTree {
    ($_) = @_;
    s/^ *//;
    if (s/^\(([^ ]+)//) {
        my $lhs = $1;
        s/^ //;
        my @c = ();
        while (1) {
            my ($child, $rest) = parseTree($_);
            $_ = $rest;
            push @c, \%{$child};
            s/ *//;
            if ($_ eq '') {  die; }
            elsif (s/^\)//) { 
                last; 
            }
        }
        my %tree = ('lhs' => $lhs, 'children' => \@c);
        return (\%tree, $_);
    } elsif (/^\)/) { 
        die;
    } elsif (s/^([^ \)]+) *//) {
        my %tree = ('lhs' => $1);
        return (\%tree, $_);
    }
}
